@@ -1,4 +1,4 @@
-'''Parse strings using a specification based on the Python format() syntax.
+r'''Parse strings using a specification based on the Python format() syntax.
 
    ``parse()`` is the opposite of ``format()``
 
@@ -11,6 +11,8 @@ From there it's a simple thing to parse a string:
 
 >>> parse("It's {}, I love it!", "It's spam, I love it!")
 <Result ('spam',) {}>
+>>> _[0]
+'spam'
 
 Or to search a string for some pattern:
 
@@ -19,14 +21,14 @@ Or to search a string for some pattern:
 
 Or find all the occurrances of some pattern in a string:
 
->>> ''.join(r.fixed[0] for r in findall(">{}<", "<p>some <b>bold</b> text</p>")
-"some bold text"
+>>> ''.join(r.fixed[0] for r in findall(">{}<", "<p>some <b>bold</b> text</p>"))
+'some bold text'
 
 If you're going to use the same pattern to match lots of strings you can
 compile it once:
 
->>> import parse
->>> p = parse.compile("It's {}, I love it!")
+>>> from parse import compile
+>>> p = compile("It's {}, I love it!")
 >>> print p
 <Parser "It's {}, I love it!">
 >>> p.parse("It's spam, I love it!")
@@ -44,7 +46,7 @@ A basic version of the `Format String Syntax`_ is supported with anonymous
 
    {[field name]:[format spec]}
 
-Field names must be a single Python identifier word. No attributes or
+Field names must be a valid Python identifiers, including dotted names;
 element indexes are supported (as they would make no sense.)
 
 Numbered fields are also not supported: the result of parsing will include
@@ -68,6 +70,19 @@ Some simple parse() format string examples:
 <Result () {'item': 'hand grenade'}>
 >>> print r.named
 {'item': 'hand grenade'}
+>>> print r['item']
+hand grenade
+
+Dotted names are possible though the application must make additional sense of
+the result:
+
+>>> r = parse("Mmm, {food.type}, I love it!", "Mmm, spam, I love it!")
+>>> print r
+<Result () {'food.type': 'spam'}>
+>>> print r.named
+{'food.type': 'spam'}
+>>> print r['food.type']
+spam
 
 
 Format Specification
@@ -138,9 +153,8 @@ does not match:
 >>> parse('Our {:d} {:w} are...', 'Our 3 weapons are...')
 <Result (3, 'weapons') {}>
 >>> parse('Our {:d} {:w} are...', 'Our three weapons are...')
-None
 >>> parse('Meet at {:tg}', 'Meet at 1/2/2011 11:00 PM')
-<Result (datetime.datetime(2011, 2, 1, 23, 00),) {}>
+<Result (datetime.datetime(2011, 2, 1, 23, 0),) {}>
 
 And messing about with alignment:
 
@@ -177,9 +191,6 @@ currently result in a resource allocation issue. A TooManyFields exception
 will be raised in this instance. The current limit is about 15. It is hoped
 that this limit will be removed one day.
 
-See also the unit tests at the end of the module for some more
-examples. Run the tests with "python -m parse".
-
 .. _`Format String Syntax`: http://docs.python.org/library/string.html#format-string-syntax
 .. _`Format Specification Mini-Language`: http://docs.python.org/library/string.html#format-specification-mini-language
 
@@ -215,16 +226,60 @@ will be substituted in the ``Result`` instance for that field.
 Your custom type conversions may override the builtin types if you supply one
 with the same identifier.
 
->>> def converter(string):
+>>> def shouty(string):
 ...    return string.upper()
 ...
->>> r = parse('{:shouty} world', 'hello world', dict(shouty=shouty))
+>>> parse('{:shouty} world', 'hello world', dict(shouty=shouty))
 <Result ('HELLO',) {}>
 
+If the type converter has the optional ``pattern`` attribute, it is used as
+regular expression for better pattern matching (instead of the default one).
+
+>>> def parse_number(text):
+...    return int(text)
+>>> parse_number.pattern = r'\d+'
+>>> parse('Answer: {number:Number}', 'Answer: 42', dict(Number=parse_number))
+<Result () {'number': 42}>
+>>> _ = parse('Answer: {:Number}', 'Answer: Alice', dict(Number=parse_number))
+>>> assert _ is None, "MISMATCH"
+
+You can also use the ``with_pattern(pattern)`` decorator to add this
+information to a type converter function:
+
+>>> from parse import with_pattern
+>>> @with_pattern(r'\d+')
+... def parse_number(text):
+...    return int(text)
+>>> parse('Answer: {number:Number}', 'Answer: 42', dict(Number=parse_number))
+<Result () {'number': 42}>
+
+A more complete example of a custom type might be:
+
+>>> yesno_mapping = {
+...     "yes":  True,   "no":    False,
+...     "on":   True,   "off":   False,
+...     "true": True,   "false": False,
+... }
+... @with_pattern(r"|".join(yesno_mapping))
+... def parse_yesno(text):
+...     return yesno_mapping[text.lower()]
+
+
 ----
 
 **Version history (in brief)**:
 
+- 1.6.0 add support for optional ``pattern`` attribute in user-defined types
+  (thanks Jens Engel)
+- 1.5.3 fix handling of question marks
+- 1.5.2 fix type conversion error with dotted names (thanks Sebastian Thiel)
+- 1.5.1 implement handling of named datetime fields
+- 1.5 add handling of dotted field names (thanks Sebastian Thiel)
+- 1.4.1 fix parsing of "0" in int conversion (thanks James Rowe)
+- 1.4 add __getitem__ convenience access on Result.
+- 1.3.3 fix Python 2.5 setup.py issue.
+- 1.3.2 fix Python 3.2 setup.py issue.
+- 1.3.1 fix a couple of Python 3.2 compatibility issues.
 - 1.3 added search() and findall(); removed compile() from ``import *``
   export as it overwrites builtin.
 - 1.2 added ability for custom and override type conversions to be
@@ -249,17 +304,48 @@ with the same identifier.
   and removed the restriction on mixing fixed-position and named fields
 - 1.0.0 initial release
 
-This code is copyright 2011 eKit.com Inc (http://www.ekit.com/)
+This code is copyright 2012 Richard Jones <richard@python.org>
 See the end of the source file for the license of use.
 '''
-__version__ = '1.3'
+__version__ = '1.6.0'
 
 # yes, I now have two problems
 import re
+import sys
 from datetime import datetime, time, tzinfo, timedelta
 from functools import partial
+import logging
 
-__all__ = 'parse search findall'.split()
+__all__ = 'parse search findall with_pattern'.split()
+
+log = logging.getLogger(__name__)
+
+
+def with_pattern(pattern):
+    """Attach a regular expression pattern matcher to a custom type converter
+    function.
+
+    This annotates the type converter with the :attr:`pattern` attribute.
+
+    EXAMPLE:
+        >>> import parse
+        >>> @parse.with_pattern(r"\d+")
+        ... def parse_number(text):
+        ...     return int(text)
+
+    is equivalent to:
+
+        >>> def parse_number(text):
+        ...     return int(text)
+        >>> parse_number.pattern = r"\d+"
+
+    :param pattern: regular expression pattern (as text)
+    :return: wrapped function
+    """
+    def decorator(func):
+        func.pattern = pattern
+        return func
+    return decorator
 
 
 def int_convert(base):
@@ -278,7 +364,7 @@ def int_convert(base):
         else:
             sign = 1
 
-        if string[0] == '0':
+        if string[0] == '0' and len(string) > 1:
             if string[1] in 'bB':
                 base = 2
             elif string[1] in 'oO':
@@ -302,6 +388,8 @@ def percentage(string, match):
 class FixedTzOffset(tzinfo):
     """Fixed offset in minutes east from UTC.
     """
+    ZERO = timedelta(0)
+
     def __init__(self, offset, name):
         self._offset = timedelta(minutes = offset)
         self._name = name
@@ -316,7 +404,7 @@ class FixedTzOffset(tzinfo):
         return self._name
 
     def dst(self, dt):
-        return ZERO
+        return self.ZERO
 
     def __eq__(self, other):
         return self._name == other._name and self._offset == other._offset
@@ -391,7 +479,7 @@ def date_convert(string, match, ymd=None, mdy=None, dmy=None,
         tz = tz.strip()
         if tz.isupper():
             # TODO use the awesome python TZ module?
-            TODO
+            pass
         else:
             sign = tz[0]
             if ':' in tz:
@@ -421,7 +509,7 @@ class TooManyFields(ValueError):
     pass
 
 # note: {} are handled separately
-REGEX_SAFETY = re.compile(r'([\\.[\]()*+\^$!])')
+REGEX_SAFETY = re.compile(r'([?\\.[\]()*+\^$!])')
 
 # allowed field types
 ALLOWED_TYPES = set(list('nbox%fegwWdDsS') +
@@ -460,11 +548,17 @@ def extract_format(format, extra_types):
     return locals()
 
 
-PARSE_RE = re.compile('({{|}}|{}|{:[^}]+?}|{\w+?}|{\w+?:[^}]+?})')
+PARSE_RE = re.compile(r'({{|}}|{}|{:[^}]+?}|{\w+?(?:\.\w+?)*}|{\w+?(?:\.\w+?)*:[^}]+?})')
 
 
 class Parser(object):
+    '''Encapsulate a format string that may be used to parse other strings.
+    '''
     def __init__(self, format, extra_types={}):
+        # a mapping of a name as in {hello.world} to a regex-group compatible name, like hello__world
+        # Its used to prevent the transformation of name-to-group and group to name to fail subtly, such
+        # as in: hello_.world-> hello___world->hello._world
+        self._group_to_name_map = {}
         self._format = format
         self._extra_types = extra_types
         self._fixed_fields = []
@@ -472,27 +566,48 @@ class Parser(object):
         self._group_index = 0
         self._type_conversions = {}
         self._expression = self._generate_expression()
-        self._search_re = None
-        self._match_re = None
+        self.__search_re = None
+        self.__match_re = None
+
+        logging.debug('format %r -> %r' % (format, self._expression))
 
     def __repr__(self):
         if len(self._format) > 20:
             return '<%s %r>' % (self.__class__.__name__, self._format[:17] + '...')
         return '<%s %r>' % (self.__class__.__name__, self._format)
 
+    @property
+    def _search_re(self):
+        if self.__search_re is None:
+            try:
+                self.__search_re = re.compile(self._expression, re.IGNORECASE|re.DOTALL)
+            except AssertionError:
+                e = sys.exc_info()[1]   # to keep py3k and backward compat
+                if str(e).endswith('this version only supports 100 named groups'):
+                    raise TooManyFields('sorry, you are attempting to parse too '
+                        'many complex fields')
+        return self.__search_re
+
+    @property
+    def _match_re(self):
+        if self.__match_re is None:
+            expression = '^%s$' % self._expression
+            try:
+                self.__match_re = re.compile(expression, re.IGNORECASE|re.DOTALL)
+            except AssertionError:
+                e = sys.exc_info()[1]   # to keep py3k and backward compat
+                if str(e).endswith('this version only supports 100 named groups'):
+                    raise TooManyFields('sorry, you are attempting to parse too '
+                        'many complex fields')
+            except re.error:
+                raise NotImplementedError("Group names (e.g. (?P<name>) can cause failure, as they are not esacped properly: '%s'" % expression)
+        return self.__match_re
+
     def parse(self, string):
         '''Match my format to the string exactly.
 
         Return either a Result instance or None if there's no match.
         '''
-        if self._match_re is None:
-            expression = '^%s$' % self._expression
-            try:
-                self._match_re = re.compile(expression, re.IGNORECASE|re.DOTALL)
-            except AssertionError, e:
-                if str(e).endswith('this version only supports 100 named groups'):
-                    raise TooManyFields('sorry, you are attempting to parse too '
-                        'many complex fields')
         m = self._match_re.match(string)
         if m is None:
             return None
@@ -508,13 +623,6 @@ class Parser(object):
 
         Return either a Result instance or None if there's no match.
         '''
-        if self._search_re is None:
-            try:
-                self._search_re = re.compile(self._expression, re.IGNORECASE|re.DOTALL)
-            except AssertionError, e:
-                if str(e).endswith('this version only supports 100 named groups'):
-                    raise TooManyFields('sorry, you are attempting to parse too '
-                        'many complex fields')
         if endpos is None:
             endpos = len(string)
         m = self._search_re.search(string, pos, endpos)
@@ -533,13 +641,6 @@ class Parser(object):
         Returns an iterator that holds Result instances for each format match
         found.
         '''
-        if self._search_re is None:
-            try:
-                self._search_re = re.compile(self._expression, re.IGNORECASE|re.DOTALL)
-            except AssertionError, e:
-                if str(e).endswith('this version only supports 100 named groups'):
-                    raise TooManyFields('sorry, you are attempting to parse too '
-                        'many complex fields')
         if endpos is None:
             endpos = len(string)
         return ResultIterator(self, string, pos, endpos)
@@ -555,14 +656,17 @@ class Parser(object):
         # grab the named fields, converting where requested
         groupdict = m.groupdict()
         named_fields = {}
+        name_map = {}
         for k in self._named_fields:
+            korig = self._group_to_name_map[k]
+            name_map[korig] = k
             if k in self._type_conversions:
-                named_fields[k] = self._type_conversions[k](groupdict[k], m)
+                named_fields[korig] = self._type_conversions[k](groupdict[k], m)
             else:
-                named_fields[k] = groupdict[k]
+                named_fields[korig] = groupdict[k]
 
         # now figure the match spans
-        spans = dict((n, m.span(n)) for n in named_fields)
+        spans = dict((n, m.span(name_map[n])) for n in named_fields)
         spans.update((i, m.span(n+1)) for i, n in enumerate(self._fixed_fields))
 
         # and that's our result
@@ -589,6 +693,24 @@ class Parser(object):
                 e.append(REGEX_SAFETY.sub(self._regex_replace, part))
         return ''.join(e)
 
+    def _to_group_name(self, field):
+        # return a version of field which can be used as capture group, even
+        # though it might contain '.'
+        group = field.replace('.', '_')
+
+        # make sure we don't collide
+        n = 1
+        while group in self._group_to_name_map:
+            n += 1
+            if '.' in field:
+                group = field.replace('.', '_'*n)
+            else:
+                group = field.replace('_', '_'*n)
+
+        # save off the mapping
+        self._group_to_name_map[group] = field
+        return group
+
     def _handle_field(self, field):
         # first: lose the braces
         field = field[1:-1]
@@ -601,9 +723,10 @@ class Parser(object):
                 name, format = field.split(':')
             else:
                 name = field
-            self._named_fields.append(name)
-            group = name
-            wrap = '(?P<%s>%%s)' % name
+            group = self._to_group_name(name)
+            self._named_fields.append(group)
+            # this will become a group, which must not contain dots
+            wrap = '(?P<%s>%%s)' % group
         else:
             self._fixed_fields.append(self._group_index)
             wrap = '(%s)'
@@ -611,7 +734,7 @@ class Parser(object):
                 format = field[1:]
             group = self._group_index
 
-        # simplest case: a bare {}
+        # simplest case: no type specifier ({} or {name})
         if not format:
             self._group_index += 1
             return wrap % '.+?'
@@ -623,9 +746,10 @@ class Parser(object):
         type = format['type']
         is_numeric = type and type in 'n%fegdobh'
         if type in self._extra_types:
-            s = '.+?'
-            def f(string, m, type=type):
-                return self._extra_types[type](string)
+            type_converter = self._extra_types[type]
+            s = getattr(type_converter, 'pattern', r'.+?')
+            def f(string, m):
+                return type_converter(string)
             self._type_conversions[group] = f
         elif type == 'n':
             s = '\d{1,3}([,.]\d{3})*'
@@ -663,59 +787,52 @@ class Parser(object):
         elif type == 'ti':
             s = r'(\d{4}-\d\d-\d\d)((\s+|T)%s)?(Z|[-+]\d\d:\d\d)?' % TIME_PAT
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, ymd=n,
-                hms=n+3, tz=n+6)
+            self._type_conversions[group] = partial(date_convert, ymd=n+1,
+                hms=n+4, tz=n+7)
             self._group_index += 7
-            wrap = ''
         elif type == 'tg':
             s = r'(\d{1,2}[-/](\d{1,2}|%s)[-/]\d{4})(\s+%s)?%s?%s?' % (
                 ALL_MONTHS_PAT, TIME_PAT, AM_PAT, TZ_PAT)
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, dmy=n,
-                hms=n+4, am=n+7, tz=n+8)
+            self._type_conversions[group] = partial(date_convert, dmy=n+1,
+                hms=n+5, am=n+8, tz=n+9)
             self._group_index += 9
-            wrap = ''
         elif type == 'ta':
             s = r'((\d{1,2}|%s)[-/]\d{1,2}[-/]\d{4})(\s+%s)?%s?%s?' % (
                 ALL_MONTHS_PAT, TIME_PAT, AM_PAT, TZ_PAT)
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, mdy=n,
-                hms=n+4, am=n+7, tz=n+8)
+            self._type_conversions[group] = partial(date_convert, mdy=n+1,
+                hms=n+5, am=n+8, tz=n+9)
             self._group_index += 9
-            wrap = ''
         elif type == 'te':
             # this will allow microseconds through if they're present, but meh
             s = r'(%s,\s+)?(\d{1,2}\s+%s\s+\d{4})\s+%s%s' % (DAYS_PAT,
                 MONTHS_PAT, TIME_PAT, TZ_PAT)
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, dmy=n+2,
-                hms=n+4, tz=n+7)
+            self._type_conversions[group] = partial(date_convert, dmy=n+3,
+                hms=n+5, tz=n+8)
             self._group_index += 8
-            wrap = ''
         elif type == 'th':
             # slight flexibility here from the stock Apache format
             s = r'(\d{1,2}[-/]%s[-/]\d{4}):%s%s' % (MONTHS_PAT, TIME_PAT,
                 TZ_PAT)
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, dmy=n,
-                hms=n+2, tz=n+5)
+            self._type_conversions[group] = partial(date_convert, dmy=n+1,
+                hms=n+3, tz=n+6)
             self._group_index += 6
-            wrap = ''
         elif type == 'tc':
             s = r'(%s)\s+%s\s+(\d{1,2})\s+%s\s+(\d{4})' % (
                 DAYS_PAT, MONTHS_PAT, TIME_PAT)
             n = self._group_index
             self._type_conversions[group] = partial(date_convert,
-                d_m_y=(n+3,n+2,n+7), hms=n+4)
+                d_m_y=(n+4,n+3,n+8), hms=n+5)
             self._group_index += 8
-            wrap = ''
         elif type == 'tt':
             s = r'%s?%s?%s?' % (TIME_PAT, AM_PAT, TZ_PAT)
             n = self._group_index
-            self._type_conversions[group] = partial(date_convert, hms=n,
-                am=n+3, tz=n+4)
+            self._type_conversions[group] = partial(date_convert, hms=n+1,
+                am=n+4, tz=n+5)
             self._group_index += 5
-            wrap = ''
         elif type:
             s = r'\%s+' % type
         else:
@@ -772,12 +889,20 @@ class Parser(object):
 
 class Result(object):
     '''The result of a parse() or search().
+
+    Fixed results may be looked up using result[index]. Named results may be
+    looked up using result['name'].
     '''
     def __init__(self, fixed, named, spans):
         self.fixed = fixed
         self.named = named
         self.spans = spans
 
+    def __getitem__(self, item):
+        if isinstance(item, int):
+            return self.fixed[item]
+        return self.named[item]
+
     def __repr__(self):
         return '<%s %r %r>' % (self.__class__.__name__, self.fixed,
             self.named)
@@ -797,13 +922,16 @@ class ResultIterator(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         m = self.parser._search_re.search(self.string, self.pos, self.endpos)
         if m is None:
             raise StopIteration()
         self.pos = m.end()
         return self.parser._generate_result(m)
 
+    # pre-py3k compat
+    next = __next__
+
 
 def parse(format, string, extra_types={}):
     '''Using "format" attempt to pull values from "string".
@@ -887,7 +1015,7 @@ def compile(format, extra_types={}):
     return Parser(format, extra_types=extra_types)
 
 
-# Copyright (c) 2011 eKit.com Inc (http://www.ekit.com/)
+# Copyright (c) 2012 Richard Jones <richard@python.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
